==> libiriver-0.0.7.tar.gz: libiriver-0.0.7/python/pyiriver.c <==
#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <Python.h>
#include "../src/iriver.h"

#include <stdio.h>
#include <usb.h>

#define TRUE  1
#define FALSE 0

typedef struct {
	PyObject_HEAD
	int initialized;
	int test;
	struct usb_dev_handle *d;
} iriverobject;	

staticforward PyTypeObject IriverObjectType;
static PyObject *IriverError;

static PyObject *progress_callback = NULL;

static void
stub_progress_callback(const char *name)
{
    PyObject *arglist = Py_BuildValue("(s)", name);
    PyObject *result  = PyEval_CallObject(progress_callback, arglist);
 
    Py_DECREF(arglist);
    return; 
}
 

static PyObject *
iriver_process_error_code(int i)
{
	if (i < 0) {
		PyErr_SetString(IriverError, ifp_error_to_string(i));
		return NULL;
	}
}

/* Instance methods */
static PyObject *
iriver_delete_file(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	PyObject *pycompobj = NULL;

	if (!PyArg_ParseTuple(args, "s|O", &s, &pycompobj)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	if ((pycompobj != NULL) && (!PyCallable_Check(pycompobj))) {
	    PyErr_SetString(PyExc_TypeError, "Need a callable object or None!");
	    return NULL;
	} 
	if (pycompobj == NULL)
		r = ifp_delete_file(self->d, s, NULL);
	else {
		progress_callback = pycompobj;
		r = ifp_delete_dir(self->d, s, stub_progress_callback);
	}
	if (r == IRIVER_OK) {
		return Py_BuildValue("b", TRUE);	
	} else 
		return(iriver_process_error_code(r));
}

static PyObject *
iriver_delete_dir(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	PyObject *pycompobj = NULL;

	if (!PyArg_ParseTuple(args, "s|O", &s, &pycompobj)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	if ((pycompobj != NULL) && (!PyCallable_Check(pycompobj))) {
	    PyErr_SetString(PyExc_TypeError, "Need a callable object or None!");
	    return NULL;
	} 
	if (pycompobj == NULL)
		r = ifp_delete_dir(self->d, s, NULL);
	else {
		progress_callback = pycompobj;
		r = ifp_delete_dir(self->d, s, stub_progress_callback);
	}
	if (r == IRIVER_OK) {
		return Py_BuildValue("b", TRUE);	
	} else 
		return(iriver_process_error_code(r));
}

static PyObject *
iriver_file_exists(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	r = ifp_file_exists(self->d, s);
	if (r == IRIVER_FILE_EXISTS) {
		return Py_BuildValue("b", TRUE);	
	} else 
		return (iriver_process_error_code(r));
}

static PyObject *
iriver_empty_dir(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	r = ifp_empty_dir(self->d, s);
	if (r == FALSE) {
		return Py_BuildValue("b", FALSE);
	} else if (r == TRUE) {
		return Py_BuildValue("b", TRUE);
	} else return(iriver_process_error_code(r));
}

static PyObject *
iriver_firmware_update(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	r = ifp_firmware_update(&(self->d), s);
	if (r == IRIVER_OK) {
		return Py_BuildValue("");
	} else return(iriver_process_error_code(r));
}

static PyObject *
iriver_format(iriverobject *self, PyObject *args)
{
	int r;
	if (!(self->initialized))
		return NULL;
	r = ifp_format(self->d);
/* FIXME: Disable checking for now. ifp_format() always returns -2. So does ifp_line. This always used to be this way? */
/*
	if (r != IRIVER_OK) {
		PyErr_SetString(IriverError, "I/O Error");
		return NULL;
	} 
*/
	return Py_BuildValue("");
}

static PyObject *
iriver_get_info(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	ifp_file_info *f = NULL;
  
	if (!PyArg_ParseTuple(args, "s", &s))
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;

	r = ifp_get_info(self->d, s, &f);
	if (r!=IRIVER_OK) return(iriver_process_error_code(r));
	/* Return "name, size, mime, is_file" */
	return Py_BuildValue("{s:s, s:i, s:s, s:b}", 
	  "name", f->name, "size", f->size, 
	  "mime", f->mimetype, "is_file", f->is_file);
}

static PyObject *
iriver_make_dir(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	r = ifp_make_dir(self->d, s);
	if (r == IRIVER_OK) {
		return Py_BuildValue("");
	} else 
		return(iriver_process_error_code(r));
}

static PyObject *
iriver_file_download(iriverobject *self, PyObject *args)
{
	int r;
	char *s, *local;
	ifp_file_info *info;
	PyObject *pycompobj = NULL;
	
	if (!PyArg_ParseTuple(args, "ss|O", &s, &local, &pycompobj)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	if ((pycompobj != NULL) && (!PyCallable_Check(pycompobj))) {
		PyErr_SetString(PyExc_TypeError, "Need a callable object or None!");
		return NULL;
	} 

	r = ifp_get_info(self->d, s, &info); /* FIXME: add proper error checking */
	if (info->is_file) {
		if (pycompobj == NULL)
			r = ifp_download_file(self->d, s, local, NULL);
		else {
			progress_callback = pycompobj;
			r = ifp_download_file(self->d, s, local, stub_progress_callback);
		}
	} else {
		local[strlen(local)-strlen(basename(s))-1]='\0';
		if (pycompobj == NULL)
			r = ifp_download_dir(self->d, s, local, NULL);
		else {
			progress_callback = pycompobj;
			r = ifp_download_dir(self->d, s, local, stub_progress_callback);
		}
	}
	if (r == IRIVER_OK) {
		return Py_BuildValue("");
	} else 
		return (iriver_process_error_code(r));
}
 
static PyObject *
iriver_file_upload(iriverobject *self, PyObject *args)
{
	int r;
	char *s, *local;
	struct stat st;
	PyObject *pycompobj = NULL;
	
	if (!PyArg_ParseTuple(args, "ss|O", &s, &local, &pycompobj)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	if ((pycompobj != NULL) && (!PyCallable_Check(pycompobj))) {
		PyErr_SetString(PyExc_TypeError, "Need a callable object or None!");
		return NULL;
	} 

	stat(local, &st);	
	if (S_ISDIR(st.st_mode)) {
		s[strlen(s) - strlen(basename(local)) - 1] = '\0';
		if (pycompobj == NULL)
			r = ifp_upload_dir(self->d, s, local, NULL);
		else {
			progress_callback = pycompobj;
			r = ifp_upload_dir(self->d, s, local, stub_progress_callback);
		}
	} else {
		if (pycompobj == NULL)
			r = ifp_upload_file(self->d, s, local, NULL);
		else {
			progress_callback = pycompobj;
			r = ifp_upload_file(self->d, s, local, stub_progress_callback);
		}
	}
	if (r == IRIVER_OK) 
		return Py_BuildValue("");
	else 
		return (iriver_process_error_code(r));
}

/* Still b0rked */
static PyObject *
iriver_ls(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	ifp_file_info *list, *tmp;
	PyObject *ret_list;
	
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	r = ifp_ls(self->d, s, &list);
	if ((r == IRIVER_OK) && (list != NULL)) {
		tmp = list;
		ret_list = PyList_New(0);
		while (tmp != NULL) {
			PyList_Append(ret_list, Py_BuildValue("{s:s, s:i, s:s, s:i}", 
			    "name", tmp->name, "size", tmp->size, "mime", tmp->mimetype, "is_file", tmp->is_file));
			tmp = tmp->next;
		}
		ifp_free_file_list(&list);	
		return Py_BuildValue("O", ret_list);
	} else if ((r == IRIVER_OK) && (list == NULL)) {
		return Py_BuildValue("");
	} else return 
		(iriver_process_error_code(r));
		
// FIXME: Why would we need this? This is only hit when the return value is != IRIVER_OK.
//	return Py_BuildValue(""); 
}

/* Instance method table */
static struct PyMethodDef iriver_methods[] = {
	{ "file_delete",     iriver_delete_file,     METH_VARARGS },
	{ "empty_dir",       iriver_empty_dir,       METH_VARARGS },
	{ "dir_delete",      iriver_delete_dir,      METH_VARARGS },
	{ "file_download",   iriver_file_download,   METH_VARARGS },
	{ "file_exists",     iriver_file_exists,     METH_VARARGS },
	{ "file_upload",     iriver_file_upload,     METH_VARARGS },
	{ "firmware_update", iriver_firmware_update, METH_VARARGS },
	{ "format",          iriver_format,          METH_VARARGS },
	{ "get_info",        iriver_get_info,        METH_VARARGS },
	{ "ls",              iriver_ls,              METH_VARARGS },
	{ "make_dir",        iriver_make_dir,        METH_VARARGS },
	{ NULL, NULL }
};

/* Basic operations */
static iriverobject *
new_iriver(int init_type)
{
	int r;
	iriverobject *self;
	self = PyObject_NEW(iriverobject, &IriverObjectType);
	if (self == NULL)
		return NULL;

	r = ifp_init(&(self->d), IRIVER_INIT);
	if (r<0) return(iriver_process_error_code(r));
	self->initialized = 1;
	return self;
}	

static void
dealloc_iriver(iriverobject *self) 
{
	ifp_release(self->d);
	self->initialized = 0;
	PyMem_DEL(self);
}

static PyObject *
iriver_getattr(iriverobject *self, char *name)
{
	int i;
	float f;
	char *s;
	
	if (strcmp(name, "initialized") == 0) {
		return Py_BuildValue("i", self->initialized);
	} else if (self->initialized && strcmp(name, "test") == 0) {
		return Py_BuildValue("i", self->test);
	} else if (self->initialized && strcmp(name, "capacity") == 0) {
		i = ifp_get_capacity(self->d);
		if (i<0) return(iriver_process_error_code(i));
		return Py_BuildValue("i", i);
	} else if (self->initialized && strcmp(name, "free") == 0) {
		i = ifp_get_free(self->d);
		return Py_BuildValue("i", i);
	} else if (self->initialized && strcmp(name, "firmware") == 0) {
		f = ifp_firmware_version(self->d);
		return Py_BuildValue("f", f);
	} else if (self->initialized && strcmp(name, "model") == 0) {
		s = (char*)malloc(60);
		memset(s, 0, sizeof(s));
		i = ifp_model_name(self->d, s);
		return Py_BuildValue("s", s);
	}

	return Py_FindMethod(iriver_methods, (PyObject *)self, name);
}

static PyObject *
iriver_repr(iriverobject *self)
{
	char rbuffer[256];
	sprintf(rbuffer, "%d iriver %d test", self->test,
	  self->initialized);
	return PyString_FromString(rbuffer);
}


static PyTypeObject IriverObjectType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,				/* ob_size */
	"Iriver",			/* tp_name */
	sizeof(iriverobject),		/* tp_basicsize */
	0,				/* tp_itemsize */

	(destructor)	dealloc_iriver,	/* tp_dealloc (refcount = 0) */
	(printfunc)	0,		/* tp_print */
	(getattrfunc)	iriver_getattr, /* tp_getattr (x.attr) */
	(setattrfunc)	0,		/* tp_setattr */
	(cmpfunc)	0,		/* tp_compare */
	(reprfunc)	iriver_repr,	/* tp_repr (repr(x) */

	0,				/* tp_as_number */
	0,				/* tp_as_sequence */
	0,				/* tp_as_mapping */
	(hashfunc)	0,		/* tp_hash (dict[x])*/
	(ternaryfunc)	0,		/* tp_call x() */
	(reprfunc)	0,		/* tp_str str(x) */
};

static PyObject *
iriverobject_new(PyObject *self, PyObject *args)
{
	return (PyObject *) new_iriver(0);
}

static struct PyMethodDef irivertype_methods[] = {
	{ "Iriver", iriverobject_new, METH_VARARGS },
	{ NULL, NULL }
};

void initiriver() {
	PyObject *m, *d;

	m = Py_InitModule("iriver", irivertype_methods);
	d = PyModule_GetDict(m);
	/* Create exception object */
	IriverError = PyErr_NewException("iriver.error", NULL, NULL);
	PyDict_SetItemString(d, "error", IriverError);
}

==> libiriver-0.0.10.tar.gz: libiriver-0.0.10/python/pyiriver.c <==
#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <Python.h>
#include "../src/iriver.h"

#include <stdio.h>
#include <usb.h>

#define TRUE  1
#define FALSE 0

typedef struct {
	PyObject_HEAD
	int initialized;
	int test;
	struct usb_dev_handle *d;
} iriverobject;	

staticforward PyTypeObject IriverObjectType;
static PyObject *IriverError;

static PyObject *progress_callback = NULL;

static void
stub_progress_callback(const char *name)
{
    PyObject *arglist = Py_BuildValue("(s)", name);
    PyObject *result  = PyEval_CallObject(progress_callback, arglist);
 
    Py_DECREF(arglist);
    return; 
}
 

static PyObject *
iriver_process_error_code(int i)
{
	if (i < 0) {
		PyErr_SetString(IriverError, ifp_error_to_string(i));
		return NULL;
	}
}

/* Instance methods */
static PyObject *
iriver_delete_file(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	PyObject *pycompobj = NULL;

	if (!PyArg_ParseTuple(args, "s|O", &s, &pycompobj)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	if ((pycompobj != NULL) && (!PyCallable_Check(pycompobj))) {
	    PyErr_SetString(PyExc_TypeError, "Need a callable object or None!");
	    return NULL;
	} 
	if (pycompobj != NULL) {
		switch (ifp_file_exists(self->d, s)) {
		case IRIVER_FILE_EXISTS:
			r = ifp_delete_file(self->d, s, NULL);
			break;
		case IRIVER_DIRECTORY_EXISTS:
			progress_callback = pycompobj;
			r = ifp_delete_dir(self->d, s, stub_progress_callback);
			break;
		default:
			break;
		}
	}

	if (r == IRIVER_OK) {
		return Py_BuildValue("b", TRUE);	
	} else 
		return(iriver_process_error_code(r));
}

static PyObject *
iriver_delete_dir(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	PyObject *pycompobj = NULL;

	if (!PyArg_ParseTuple(args, "s|O", &s, &pycompobj)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	if ((pycompobj != NULL) && (!PyCallable_Check(pycompobj))) {
	    PyErr_SetString(PyExc_TypeError, "Need a callable object or None!");
	    return NULL;
	} 
	if (pycompobj == NULL)
		r = ifp_delete_dir(self->d, s, NULL);
	else {
		progress_callback = pycompobj;
		r = ifp_delete_dir(self->d, s, stub_progress_callback);
	}
	if (r == IRIVER_OK) {
		return Py_BuildValue("b", TRUE);	
	} else 
		return(iriver_process_error_code(r));
}

static PyObject *
iriver_file_exists(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	r = ifp_file_exists(self->d, s);
	if (r == IRIVER_FILE_EXISTS) {
		return Py_BuildValue("b", TRUE);	
	} else 
		return (iriver_process_error_code(r));
}

static PyObject *
iriver_empty_dir(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	r = ifp_empty_dir(self->d, s);
	if (r == FALSE) {
		return Py_BuildValue("b", FALSE);
	} else if (r == TRUE) {
		return Py_BuildValue("b", TRUE);
	} else return(iriver_process_error_code(r));
}

static PyObject *
iriver_firmware_update(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	r = ifp_firmware_update(&(self->d), s);
	if (r == IRIVER_OK) {
		return Py_BuildValue("");
	} else return(iriver_process_error_code(r));
}

static PyObject *
iriver_format(iriverobject *self, PyObject *args)
{
	int r;
	if (!(self->initialized))
		return NULL;
	r = ifp_format(self->d);
/* FIXME: Disable checking for now. ifp_format() always returns -2. So does ifp_line. This always used to be this way? */
/*
	if (r != IRIVER_OK) {
		PyErr_SetString(IriverError, "I/O Error");
		return NULL;
	} 
*/
	return Py_BuildValue("");
}

static PyObject *
iriver_get_info(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	ifp_file_info *f = NULL;
  
	if (!PyArg_ParseTuple(args, "s", &s))
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;

	r = ifp_get_info(self->d, s, &f);
	if (r!=IRIVER_OK) return(iriver_process_error_code(r));
	/* Return "name, size, mime, is_file" */
	return Py_BuildValue("{s:s, s:i, s:s, s:b}", 
	  "name", f->name, "size", f->size, 
	  "mime", f->mimetype, "is_file", f->is_file);
}

static PyObject *
iriver_make_dir(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	
	r = ifp_make_dir(self->d, s);
	if (r == IRIVER_OK) {
		return Py_BuildValue("");
	} else 
		return(iriver_process_error_code(r));
}

static PyObject *
iriver_file_download(iriverobject *self, PyObject *args)
{
	int r;
	char *s, *local;
	ifp_file_info *info;
	PyObject *pycompobj = NULL;
	
	if (!PyArg_ParseTuple(args, "ss|O", &s, &local, &pycompobj)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	if ((pycompobj != NULL) && (!PyCallable_Check(pycompobj))) {
		PyErr_SetString(PyExc_TypeError, "Need a callable object or None!");
		return NULL;
	} 

	r = ifp_get_info(self->d, s, &info); /* FIXME: add proper error checking */
	if (info->is_file) {
		if (pycompobj == NULL)
			r = ifp_download_file(self->d, s, local, NULL);
		else {
			progress_callback = pycompobj;
			r = ifp_download_file(self->d, s, local, stub_progress_callback);
		}
	} else {
		local[strlen(local)-strlen(basename(s))-1]='\0';
		if (pycompobj == NULL)
			r = ifp_download_dir(self->d, s, local, NULL);
		else {
			progress_callback = pycompobj;
			r = ifp_download_dir(self->d, s, local, stub_progress_callback);
		}
	}
	if (r == IRIVER_OK) {
		return Py_BuildValue("");
	} else 
		return (iriver_process_error_code(r));
}
 
static PyObject *
iriver_file_upload(iriverobject *self, PyObject *args)
{
	int r;
	char *s, *local;
	struct stat st;
	PyObject *pycompobj = NULL;
	
	if (!PyArg_ParseTuple(args, "ss|O", &s, &local, &pycompobj)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	if ((pycompobj != NULL) && (!PyCallable_Check(pycompobj))) {
		PyErr_SetString(PyExc_TypeError, "Need a callable object or None!");
		return NULL;
	} 

	stat(local, &st);	
	if (S_ISDIR(st.st_mode)) {
		s[strlen(s) - strlen(basename(local)) - 1] = '\0';
		if (pycompobj == NULL)
			r = ifp_upload_dir(self->d, s, local, NULL);
		else {
			progress_callback = pycompobj;
			r = ifp_upload_dir(self->d, s, local, stub_progress_callback);
		}
	} else {
		if (pycompobj == NULL)
			r = ifp_upload_file(self->d, s, local, NULL);
		else {
			progress_callback = pycompobj;
			r = ifp_upload_file(self->d, s, local, stub_progress_callback);
		}
	}
	if (r == IRIVER_OK) 
		return Py_BuildValue("");
	else 
		return (iriver_process_error_code(r));
}

/* Still b0rked */
static PyObject *
iriver_ls(iriverobject *self, PyObject *args)
{
	int r;
	char *s;
	ifp_file_info *list, *tmp;
	PyObject *ret_list;
	
	if (!PyArg_ParseTuple(args, "s", &s)) 
		return NULL;
	if ((self->initialized) && (strlen(s) < 1))
		return NULL;
	r = ifp_ls(self->d, s, &list);
	if ((r == IRIVER_OK) && (list != NULL)) {
		tmp = list;
		ret_list = PyList_New(0);
		while (tmp != NULL) {
			PyList_Append(ret_list, Py_BuildValue("{s:s, s:i, s:s, s:i}", 
			    "name", tmp->name, "size", tmp->size, "mime", tmp->mimetype, "is_file", tmp->is_file));
			tmp = tmp->next;
		}
		ifp_free_file_list(&list);	
		return Py_BuildValue("O", ret_list);
	} else if ((r == IRIVER_OK) && (list == NULL)) {
		return Py_BuildValue("");
	} else return 
		(iriver_process_error_code(r));
		
// FIXME: Why would we need this? This is only hit when the return value is != IRIVER_OK.
//	return Py_BuildValue(""); 
}

/* Instance method table */
static struct PyMethodDef iriver_methods[] = {
	{ "file_delete",     (PyCFunction)iriver_delete_file,     METH_VARARGS },
	{ "empty_dir",       (PyCFunction)iriver_empty_dir,       METH_VARARGS },
	{ "dir_delete",      (PyCFunction)iriver_delete_dir,      METH_VARARGS },
	{ "file_download",   (PyCFunction)iriver_file_download,   METH_VARARGS },
	{ "file_exists",     (PyCFunction)iriver_file_exists,     METH_VARARGS },
	{ "file_upload",     (PyCFunction)iriver_file_upload,     METH_VARARGS },
	{ "firmware_update", (PyCFunction)iriver_firmware_update, METH_VARARGS },
	{ "format",          (PyCFunction)iriver_format,          METH_VARARGS },
	{ "get_info",        (PyCFunction)iriver_get_info,        METH_VARARGS },
	{ "ls",              (PyCFunction)iriver_ls,              METH_VARARGS },
	{ "make_dir",        (PyCFunction)iriver_make_dir,        METH_VARARGS },
	{ NULL, NULL }
};

/* Basic operations */
static iriverobject *
new_iriver(int init_type)
{
	int r;
	iriverobject *self;
	self = PyObject_NEW(iriverobject, &IriverObjectType);
	if (self == NULL)
		return NULL;

	r = ifp_init(&(self->d), IRIVER_INIT);
	if (r<0) {
		PyErr_SetString(IriverError, ifp_error_to_string(r));
		return NULL;
	}
	self->initialized = 1;
	return self;
}	

static void
dealloc_iriver(iriverobject *self) 
{
	ifp_release(self->d);
	self->initialized = 0;
	PyMem_DEL(self);
}

static PyObject *
iriver_getattr(iriverobject *self, char *name)
{
	int i;
	float f;
	char *s;
	
	if (strcmp(name, "initialized") == 0) {
		return Py_BuildValue("i", self->initialized);
	} else if (self->initialized && strcmp(name, "test") == 0) {
		return Py_BuildValue("i", self->test);
	} else if (self->initialized && strcmp(name, "capacity") == 0) {
		i = ifp_get_capacity(self->d);
		if (i<0) return(iriver_process_error_code(i));
		return Py_BuildValue("i", i);
	} else if (self->initialized && strcmp(name, "free") == 0) {
		i = ifp_get_free(self->d);
		return Py_BuildValue("i", i);
	} else if (self->initialized && strcmp(name, "firmware") == 0) {
		f = ifp_firmware_version(self->d);
		return Py_BuildValue("f", f);
	} else if (self->initialized && strcmp(name, "model") == 0) {
		s = (char*)malloc(60);
		memset(s, 0, sizeof(s));
		i = ifp_model_name(self->d, s);
		return Py_BuildValue("s", s);
	}

	return Py_FindMethod(iriver_methods, (PyObject *)self, name);
}

static PyObject *
iriver_repr(iriverobject *self)
{
	char rbuffer[256];
	sprintf(rbuffer, "%d iriver %d test", self->test,
	  self->initialized);
	return PyString_FromString(rbuffer);
}


static PyTypeObject IriverObjectType = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,				/* ob_size */
	"Iriver",			/* tp_name */
	sizeof(iriverobject),		/* tp_basicsize */
	0,				/* tp_itemsize */

	(destructor)	dealloc_iriver,	/* tp_dealloc (refcount = 0) */
	(printfunc)	0,		/* tp_print */
	(getattrfunc)	iriver_getattr, /* tp_getattr (x.attr) */
	(setattrfunc)	0,		/* tp_setattr */
	(cmpfunc)	0,		/* tp_compare */
	(reprfunc)	iriver_repr,	/* tp_repr (repr(x) */

	0,				/* tp_as_number */
	0,				/* tp_as_sequence */
	0,				/* tp_as_mapping */
	(hashfunc)	0,		/* tp_hash (dict[x])*/
	(ternaryfunc)	0,		/* tp_call x() */
	(reprfunc)	0,		/* tp_str str(x) */
};

static PyObject *
iriverobject_new(PyObject *self, PyObject *args)
{
	return (PyObject *) new_iriver(0);
}

static struct PyMethodDef irivertype_methods[] = {
	{ "Iriver", iriverobject_new, METH_VARARGS },
	{ NULL, NULL }
};

void initiriver() {
	PyObject *m, *d;

	m = Py_InitModule("iriver", irivertype_methods);
	d = PyModule_GetDict(m);
	/* Create exception object */
	IriverError = PyErr_NewException("iriver.error", NULL, NULL);
	PyDict_SetItemString(d, "error", IriverError);
}
